pub async fn execute_continuation_async<E: EnvelopeCodec + Clone + 'static>(
continuation: &WorkflowContinuation,
input: Bytes,
envelope_codec: &E,
) -> Result<Bytes, RuntimeError>Expand description
Execute a workflow continuation asynchronously with parallel branch execution.
Uses the func from each task in the continuation for execution, and spawns
branches in parallel using tokio tasks.
Arguments
continuation- The workflow continuation to executeinput- Input bytes for the first task
Errors
Returns an error if task execution fails.